import boto3
import datetime
import logging
import sys
from dateutil.parser import *
from datetime import timedelta

"""
This function lists all stale packer instances and terminates them. 
"""

# Define the Tag of the packer instances
PACKER_TAG = 'Packer Builder'
REGION = 'eu-west-1'
DRYRUN = False
# Define the Connection
ec2 = boto3.resource('ec2', region_name=REGION)

date = datetime.datetime.now()

logger = logging.getLogger()
logger.setLevel(logging.INFO)

ch = logging.StreamHandler(sys.stdout)
ch.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)

logger.info('Packer Tag: %s', PACKER_TAG)

def lambda_handler (event, context):
  logger.info('Starting scan for instances')

  instances = ec2.instances.filter(Filters=[
      {'Name': 'tag:Name', 'Values': [PACKER_TAG]},
      {'Name': 'instance-state-name', 'Values': ['running']},
  ])

  for instance in instances:
    instance_tags = tags_dict(instance.tags)
    instance_launch_time = instance.launch_time
    logger.info('Found New Instance, ID: [%s]', instance.id)
    logger.info('[%s]: Tags: %s', instance.id, instance_tags)
    logger.info('[%s]: Launch Time: %s', instance.id, instance_launch_time)

    current_time = datetime.datetime.now(instance_launch_time.tzinfo)
    logger.debug('Current Time: %s', current_time)

    runtime = time_diff(instance_launch_time, current_time)
    runtime_days = runtime.days
    runtime_minutes = runtime.seconds/60
    logger.info('[%s]: Runtime: %s days', instance.id, runtime_days)
    logger.info('[%s]: Runtime: %s mins', instance.id, runtime_minutes)

    if (runtime_days >= 1) or (runtime_minutes >= 120):

      logger.info('[%s]: Instance has been running for %s days and %s mins. Marking for termination', instance.id, runtime.days, runtime_minutes)
      instance.terminate(DryRun=DRYRUN)
      logger.info('[%s]: Stopped Instance', instance.id)
  return True

def time_diff(launch_time, current_time):
  running_time = current_time - launch_time
  return running_time

def tags_dict(tags):
  '''Flatten a list of tags dicts into a single dict'''
  return {t['Key']: t['Value'] for t in tags} if tags else {}